package epsilonStandalone

import java.net.URI
import java.net.URISyntaxException
import java.util.List
import org.eclipse.epsilon.common.parse.problem.ParseProblem
import org.eclipse.epsilon.common.util.StringProperties
import org.eclipse.epsilon.emc.emf.EmfModel
import org.eclipse.epsilon.eol.EolModule
import org.eclipse.epsilon.eol.IEolModule
import org.eclipse.epsilon.eol.exceptions.EolRuntimeException
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException
import org.eclipse.epsilon.eol.execute.context.Variable
import org.eclipse.epsilon.eol.models.IModel
import org.eclipse.epsilon.eol.models.IRelativePathResolver
import org.eclipse.epsilon.etl.EtlModule
import java.net.URL
import java.io.File
import org.eclipse.epsilon.egl.EglTemplateFactoryModuleAdapter
import org.eclipse.epsilon.egl.EglFileGeneratingTemplateFactory

class EpsilonBase {

	/*
	 * Reads model from file given by URI and applies EOL script and returns the modified model without storing it to the disk
	 */
	public static def EmfModel applyEolScript(
		String modelName,
		String modelURI,
		String metaModelURI, 
		String eolScriptPath, 
		List<Variable> eolParameters)
	{
		var model = createEmfModel(modelName, modelURI, metaModelURI, true, false);
		return applyEolScript(model, eolScriptPath, eolParameters)
	}
	/*
	 * Applies EOL script on Emf model and returns the modified model without storing it to the disk
	 */
	public static def EmfModel applyEolScript(
		EmfModel model, 
		String eolScriptPath, 
		List<Variable> eolParameters)
	{
		prepareAndExecute(new EolModule(), eolScriptPath, newLinkedList(model), eolParameters);
		return model;
	}
	
	public static def Object applyEtlScript(
		EmfModel input,
		EmfModel output, 
		String etlScriptPath, 
		List<Variable> eolParameters)
	{	
		prepareAndExecute(new EtlModule(), etlScriptPath, newLinkedList(input, output), eolParameters);
		return output;		
	}

	public static def String applyEglScript(
		EmfModel model, 
		String eglScriptPath, 
		List<Variable> eglParameters)
	{	
		var eglModule = new EglTemplateFactoryModuleAdapter(new EglFileGeneratingTemplateFactory());
		var file_contents = prepareAndExecute(eglModule, eglScriptPath, newLinkedList(model), eglParameters)
		return file_contents as String	
	}
	
	/*
	 * Executes Epsilon code over models 
	 */
	private static def Object prepareAndExecute(IEolModule module, String source, List<IModel> models, List<Variable> parameters) throws Exception {

		var resURI = getFileURI(source);
    	module.parse(resURI);
    	
    	if (module.getParseProblems().size() > 0) {
			System.err.println("Parse errors occurred...");
			for (ParseProblem problem : module.getParseProblems()) {
        		System.err.println(problem.toString());
      		}
      		return null;
    	}
    
    	for (IModel model : models) {
			module.getContext().getModelRepository().addModel(model);
    	}
    
    	for (Variable parameter : parameters) {
      		module.getContext().getFrameStack().put(parameter);
    	}
    
    	var result = execute(module);
    
    
    	for (IModel model : models) {
    		module.getContext().getModelRepository().removeModel(model)	
    	}
//    	module.getContext().getModelRepository().dispose(); //this prohibits executing multiple transformations in one run
    	return result;
	}
	
	private static def Object execute(IEolModule module) throws EolRuntimeException {
		return module.execute();
  	}
  	

 
   	/*
  	 * Reads emf model object from file
  	 */  	
	public static def EmfModel readEmfModel(
		String name, 
		String modelURI, 
		String metamodelURI)  
		throws EolModelLoadingException, URISyntaxException 
	{
    	var EmfModel emfModel = new EmfModel();
    	var StringProperties properties = new StringProperties();
    	properties.put(EmfModel.PROPERTY_NAME, name);
    	properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, getFileURI(metamodelURI) );
    	properties.put(EmfModel.PROPERTY_MODEL_URI, getFileURI(modelURI)); 
    	properties.put(EmfModel.PROPERTY_READONLOAD, true + "");
    	properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, false + "");
    	emfModel.load(properties, null as IRelativePathResolver);
    	return emfModel;
  	}	
  	
  	/*
  	 * Creates emf model object if a file addressed over uri does not exist
  	 */  	
	public static def EmfModel createEmfModel(
		String name, 
		String modelURI, 
		String metamodelURI, 
		boolean readOnLoad, 
		boolean storeOnDisposal)  
		throws EolModelLoadingException, URISyntaxException 
	{
    	var EmfModel emfModel = new EmfModel();
    	var StringProperties properties = new StringProperties();
    	properties.put(EmfModel.PROPERTY_NAME, name);
    	properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, getFileURI(metamodelURI) );
    	// models to be read require getFileURI(modelURI) - file must exist
    	// models to be written require just modelURI - file does not have to exist
    	properties.put(EmfModel.PROPERTY_MODEL_URI, 
    		try{ 
    			getFileURI(modelURI)
    		}
    		catch(Exception e){
    			modelURI
    		}
    	);
    	properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
    	properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
    	emfModel.load(properties, null as IRelativePathResolver);
    	return emfModel;
  	}
 

 	public static def EmfModel createEmfModelByURI(
		String name, 
		String modelURI, 
		String metamodelURI, 
		boolean readOnLoad, 
		boolean storeOnDisposal)  
		throws EolModelLoadingException, URISyntaxException 
	{
    	var EmfModel emfModel = new EmfModel();
    	var StringProperties properties = new StringProperties();
    	properties.put(EmfModel.PROPERTY_NAME, name);
    	properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, getFileURI(metamodelURI) );
    	// models to be read require getFileURI(modelURI) - file must exist
    	// models to be written require just modelURI - file does not have to exist
    	properties.put(EmfModel.PROPERTY_MODEL_URI, modelURI);
    	properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
    	properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
    	emfModel.load(properties, null as IRelativePathResolver);
    	return emfModel;
  	}

	public static def EmfModel createEmfModelByMetaModelURI(String name, String model, 
	      String metamodel, boolean readOnLoad, boolean storeOnDisposal) 
	          throws EolModelLoadingException, URISyntaxException {
	    var EmfModel emfModel = new EmfModel();
	    var StringProperties properties = new StringProperties();
	    properties.put(EmfModel.PROPERTY_NAME, name);
	    properties.put(EmfModel.PROPERTY_METAMODEL_URI, metamodel);
	    properties.put(EmfModel.PROPERTY_MODEL_URI, 
	        getFileURI(model).toString());
	    properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
	    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, 
	        storeOnDisposal + "");
	    emfModel.load(properties, null as IRelativePathResolver);
	    return emfModel;
	  }
  	
	private static def URI getFileURI(String fileName) throws URISyntaxException {	
		println("searching URI for file: "+fileName)
		
//		var URI directURI = null
//		try{
//			directURI = new URI(fileName)
//			// this will thorw if file does not exist
//			var File f = new File(directURI);
//			println( "returning directURI: "+directURI)
//			return directURI 
//		}
//		catch(Exception e) {}
		
		var URI binUri = null
		var URL res = null
		try {
			res = typeof(EpsilonBase).getResource(fileName)
		}
		catch(Exception e) {}	
		if(res==null){
			try {
				res = new URL(fileName)
			}
			catch(Exception e) {}
		}
		if(res==null){
			try {
				res = new URL("file:"+fileName)
			}
			catch(Exception e) {}
		}
		println("found resource: "+res)
		binUri = res.toURI()
		println("binUri: "+binUri)
		
		var URI uri = null
	
		if (binUri.toString().indexOf("bin") > -1) {
	  		uri = new URI(binUri.toString().replaceAll("bin", "src"));
    	}
    	else {
      		uri = binUri;
    	}
    	println("\t... resulted in: "+uri)
    	return uri;
	}
}